/// <reference types="node" />
export declare class BufferReader {
    private offset;
    private buffer;
    private encoding;
    constructor(offset?: number);
    setBuffer(offset: number, buffer: Buffer): void;
    int16(): number;
    byte(): number;
    int32(): number;
    uint32(): number;
    string(length: number): string;
    cstring(): string;
    bytes(length: number): Buffer;
}
